#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <stdio.h>
#include <stdlib.h>
#include <string>

using namespace std;

void setColor(int idischarge, int& red, int& green, int& blue);

int main(void)
{
	cout << "Reading parameters for runoff simulation." << endl;
	ifstream finParam("../param.dat");
	if(!finParam){
		cout << "param.dat cannot open." << endl;
		return 1;
	}

	// vZ̈ simulation range
	int wCol,wRow, wTopoScale;
	double wSWLON, wSWLAT;

	finParam >> wTopoScale;
	cout << "  Topography scale : " << wTopoScale << endl;

	finParam >> wCol; 
	finParam >> wRow; 
	cout << "  Number of col : " << wCol << "  Number of row : " << wRow << endl;

	finParam >> wSWLON; 
	finParam >> wSWLAT;
	cout << " Sorth West corner Longitude (degree) : " << wSWLON 
		 << " Latitude (deg) : " << wSWLAT << endl;

	const int Col = wCol;     // column number of study area
	const int Row = wRow;     // row number of study area
	const double SWLON = wSWLON; // South west corner of longitude (degree)
	const double SWLAT = wSWLAT; // South west corner of latiitude (degree)

	// ̓f[^̈ input data range
	cout << "2) Reading parameters for input data." << endl;

	int wROFnum_row, wROFnum_col;
	finParam >> wROFnum_col;
	finParam >> wROFnum_row;
	cout << "  Number of col : " << wROFnum_col 
		 << "  Number of row : " << wROFnum_row << endl;

	int wpO_x, wpO_y;
	finParam >> wpO_x;
	finParam >> wpO_y;
	cout << "  Origin column number : " << wpO_x 
		 << " row number : " << wpO_y << endl;

	float wROF_WLON;
	finParam >> wROF_WLON; 
	cout << "  The longitude of the center of input grid at the southwest corner : " 
		<< wROF_WLON << endl;

	float wROF_WLAT;
	finParam >> wROF_WLAT; 
	cout << "  The latitude of the center of input grid at the southwest corner : " 
		<< wROF_WLAT << endl;

	int wInputData_Time_Step;
	finParam >> wInputData_Time_Step; 
	cout << "  time interval of input data [sec] : " 
		 << wInputData_Time_Step << endl;
	
	const int ROFnum_col = wROFnum_col;
	const int ROFnum_row = wROFnum_row; 
	const int pO_x = wpO_x;
	const int pO_y = wpO_y;
	const float ROF_WLON = wROF_WLON;
	const float ROF_WLAT = wROF_WLAT;
	const int InputData_Time_Step = wInputData_Time_Step;

	// vZ simulation term
	cout << "3) Reading parameters for simuration term." << endl;
	int wstartYear, wendYear, wstartMonth, wendMonth;
	finParam >> wstartYear;
	finParam >> wendYear;
	finParam >> wstartMonth;
	finParam >> wendMonth;
	cout << "  Start year : " << wstartYear << "  Start month : " << wstartMonth << endl;
	cout << "  End year : " << wendYear << "  End month : " << wendMonth << endl;

	const int startYear = wstartYear; // vZJnN start year
	const int endYear   = wendYear; // vZIN end year

	if (startYear > endYear) {
		cout << "Simulation year setting is wrong" << endl;
		exit(1);
	}

	const int startMonth = wstartMonth;   // vZJn start month
	const int endMonth   = wendMonth;  // vZI end month

	finParam.close();
	//--------------------- end parameter reading ------------------------------------------------

	const int dayMonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	char DischargeFileName[256];

	int iyear, imonth;
	cout << "Input year!" << endl;
	cin >> iyear;
	cout << "Input month!" << endl;
	cin >> imonth;

	ifstream FileDaily; // Daily data

	//sprintf_s(DischargeFileName, 256, "197912_dailyDischarge.bin");
	sprintf(DischargeFileName, "./%04d%02d_dailyDischarge.bin", iyear, imonth);

	cout << "Read " << DischargeFileName << endl;
	FileDaily.open(DischargeFileName, ios::in | ios::binary);

	if(!FileDaily){
		cout << "Daily discharge data cannot open." << endl;
		return 1;
	}

	int max_time_counter = dayMonth[imonth-1];
	if (max_time_counter > 10000) {
		cout << "Max_time_counter exceeds 10000!" << endl;
		return 1;
	}

	float discharge_min, discharge_max, discharge;
	string ss;
	cout << endl;
	cout << "Do you want to set the maximum and minimun values of rainfall intensity? (y/n)" << endl;
	cin >> ss;
	if (ss == "y") {
		cout << "Input maximum value : ";
		cin >> discharge_max;
		cout << "Input minimum value : ";
		cin >> discharge_min;
		if (discharge_min >= discharge_max) {
			cout << "Wrong values inputted!" << endl;
			return 1;
		}
	}
	else {
		discharge_max = -1.0; discharge_min = 10000.0; // Find max and min value of discharge.
		for (int i = 1; i <= max_time_counter; i++) {
			for (int j = 1; j <= Row; j++) {
				for (int k = 1; k <= Col; k++) {
					FileDaily.read((char*)&discharge, sizeof(float));
					if (discharge < 0.0) {
						continue;
					}
					if (discharge > discharge_max) {
						discharge_max = discharge;
					}
					if (discharge < discharge_min) {
						discharge_min = discharge;
					}
				}
			}
		}
	}
	FileDaily.close();

    if (discharge_max <= discharge_min) {
		cout << "Maximun discharge <= Minimum discharge!" << endl;
		return 1;
	}

	cout << "Maximun discharge = " << discharge_max << " Minimum discharge = " << discharge_min << endl;

	// Daily output data re-open. 
	sprintf(DischargeFileName, "./%04d%02d_dailyDischarge.bin", iyear, imonth);
	cout << "Read " << DischargeFileName << endl;
	FileDaily.open(DischargeFileName, ios::in | ios::binary);

	if(!FileDaily){
		cout << "Discharge data cannot open." << endl;
		cout << endl;
		cout << "Program ends. Hit any key!" << endl;
		char ch;
		cin.get(ch);
		return 1;
	}

	int idischarge;
	int** mdischarge;
	mdischarge = new int*[Row];
	for(int i=0; i<Row; i++){
		mdischarge[i] = new int[Col];
	}
	int red, green, blue, pixelCol=5, pixelRow=5;
	double work;

	for (int i = 1; i <= max_time_counter; i++) {

		ofstream ppmFile;
		sprintf_s(DischargeFileName, 256, "./image/%04d%02d_%4d_discharge.ppm", iyear, imonth, i);
		cout << "Write " << DischargeFileName << "\r";

		ppmFile.open(DischargeFileName, ios::out | ios::binary);

		if(!ppmFile){
			cout << "Image output file cannot open." << endl;
			return 1;
		}
		
		ppmFile << "P6" << "\n"; // Write header information.
		ppmFile << Col*pixelCol << " " << Row*pixelRow << "\n";
		ppmFile << "255" << "\n";

		for (int j = 0; j < Row; j++) {
			for (int k = 0; k < Col; k++) {
				FileDaily.read((char*)&discharge, sizeof(float));

				if (j==1 || j==Row-2 || k==1 || k==Col-2) { // out of study area
					mdischarge[j][k] = -999;
				}
				else if (discharge < 0.0) { // out of study area
					mdischarge[j][k] = -999;
				}
				else {
					work=pow((discharge-discharge_min)/(discharge_max-discharge_min), (float)0.5);
					mdischarge[j][k] = (int)(work*255);
				}
			}
		}

		for (int j=Row-1; j >= 0; j--) {
			for (int jx=0; jx < pixelCol; jx++) {
				for (int k = 0; k < Col; k++) {
					idischarge = mdischarge[j][k];
	
					// 256 kind of colors setting.
					setColor(idischarge, red, green, blue);
	
					// each grid corresponds to pixel * pixel.
					for (int kx=0; kx < pixelCol; kx++) {
						ppmFile.put((char)red); // Red
						ppmFile.put((char)green); // Green
						ppmFile.put((char)blue); // Blue
					}
				}
			}
		}
		ppmFile.close();
	}

	cout << endl;
	cout << "Program ends!" << endl;
	//cout << "Program ends. Hit any key!" << endl;
	//char ch;
	//cin.get(ch);
}


void setColor(int input, int& red, int& green, int& blue)
{
	if (input < 0) { // blue
		red = 0;
		green = 0;
		blue = 0;
	}
	else if (0 <= input && input <= 63) { // From blue  to right blue.
		red = 0;
		green = input*4;
		blue = 252;
	}
	else if (64 <= input && input <= 127) { // From light blue to green.
		red = 0;
		green = 252;
		blue = 252 - (input-64)*4;
	}
	else if (128 <= input && input <= 191) { // From green to yellow.
		red = (input - 128)*4;
		green = 252;
		blue = 0;
	}
	else if (192 <= input && input <= 255) { // From yellow to red.
		red = 252;
		green = 252 - (input-192)*4;
		blue = 0;
	}
	else if (255 < input) { // red.
		red = 252;
		green = 252;
		blue = 0;
	}
}
